@@ -138,7 +138,8 @@ class MySocket:
             return line
         data = self.conn.recv(self.MSGLEN)
         msg = data.decode(self.ENCODING)
-        while (len(data) == self.MSGLEN and msg[-1] != self.NL) or msg.strip() == '':
+        # while (len(data) == self.MSGLEN and msg[-1] != self.NL) or msg.strip() == '':
+        while msg[-1] != self.NL or msg.strip() == '':
             data = self.conn.recv(self.MSGLEN)
             if len(data) == 0: # Connection closed
                 # @warning: even in this case we should continue, as there could be previous read lines
